#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <string.h>

#define LN 100

void hammer(double *x, double *y)
{
  double gm, t, p;
  
  t = *y * M_PI/180;
  p = *x * M_PI/180;

  gm = sqrt(2.0/(1.0 + cos(t)*cos(p/2)));
  *x = 2*gm*cos(t)*sin(p/2);
  *y = gm*sin(t);
}

void latitude_line(double t, double *lx, double *ly)
{
  double gm, p;
  int i;

  t = t * M_PI/180;
  for (i = 0 ; i <= LN; i++) {
    p = 2*M_PI/LN*i - M_PI;
    gm = sqrt(2.0/(1.0 + cos(t)*cos(p/2)));
    lx[i] = 2*gm*cos(t)*sin(p/2);
    ly[i] = gm*sin(t);
  }
}

int main(void)
{
  double x, y, hx[LN+1], hy[LN+1];
  char buf[81];
  int i, j;
  FILE *GPLT;

  GPLT = popen("gnuplot","w");
  
  fprintf(GPLT, 
    "set size ratio -1; set nokey;"
    "unset border; unset xtics; unset ytics;"
    "set xrange[-sqrt(8):sqrt(8)];"
    "set parametric; set trange[0:2*pi];"
    "R=sqrt(2);\n");

  fprintf(GPLT, "plot "
    "2*R*cos(t),R*sin(t) lt 5," 
    " '-' w l lt 3 lw 2,"
    "2*R*cos(t),0 lt 0 lw 1");

  for (j = 1 ; j <= 5; j++){
    fprintf(GPLT, ",'-' w l lt 0 lw 1");
    fprintf(GPLT, ",'-' w l lt 0 lw 1");
  }
  fflush(GPLT);
  while(fgets(buf, 80, stdin) != NULL) {
    if (strlen(buf) < 2 || buf[0] == '#') {
      fprintf(GPLT, "\n");
    } else {
      sscanf(buf, "%lf %lf", &x, &y);
      hammer(&x, &y);
      fprintf(GPLT, "%8.4f  %8.4f\n", x, y);
    }
  }
  fprintf(GPLT, "e\n");
  for (j = 1; j <= 5; j++) {
    latitude_line(j*15, hx, hy);
    for (i = 0; i <= LN; i++) {
      fprintf(GPLT, "%8.4f  %8.4f\n", hx[i], hy[i]);
    }
    fprintf(GPLT, "e\n");
    for (i = 0; i <= LN; i++) {
      fprintf(GPLT, "%8.4f  %8.4f\n", hx[i], -hy[i]);
    }
    fprintf(GPLT, "e\n");
  }

  fprintf(GPLT, "pause 30\n");
  fflush(GPLT);
  pclose(GPLT);

  return 0;
}
