#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <string.h>

double c1[2][6] = {
    { 0.609651, 0.000743349, 0.399327, 
      0.00115617 , 0.000828242, 0.000433139 },
    { 0.413292, 68.2347, -101.968, 
      41.7354, -8.16021, 0.893083 } };

void mollweide(double *xs, double *ys) 
/* xs, *ys in [rad] */
{
  double *c, y, th;

  y = *ys;
  if ( y <= 7*M_PI/16 ) c = c1[0]; 
    else c = c1[1];

  th = c[0]*asin(2*y/M_PI) + c[1] + c[2]*y 
       + c[3]*(2*y*y -1) + c[4]*(4*y*y -3)*y 
       +c[5]*(8*y*y*y*(y - 1) + 1) ;

  *xs = sqrt(8)/M_PI*(*xs)*cos(th);
  *ys = sqrt(2)*sin(th);
}

int main(void)
{
  double x, y, r, xp, yp;
  char buf[81];
  int i;
  FILE *GPLT;

  GPLT = popen("gnuplot","w");
  
  fprintf(GPLT, 
    "set size ratio -1; set nokey;"
    "unset border; unset xtics; unset ytics;"
    "set xrange[-sqrt(8):sqrt(8)];"
    "set parametric; set trange[0:2*pi];"
    "R=sqrt(2);\n"
    "plot ");

  for (i = 0 ; i <= 6; i++) {
    r = i/6.0;
    fprintf(GPLT, "2*%f*R*cos(t),R*sin(t) lt 0,", r, r);
  }
  fprintf(GPLT, " '-' w l lt 0 lw 0.5, '-' w filledcurves lt 3 lw 2\n" );
  for (i = 0; i < 6; i++) {
    xp = M_PI; 
    yp = i * M_PI/12;
    mollweide(&xp, &yp);
    fprintf(GPLT, "%f %f\n%f %f\n\n", -xp, yp, xp, yp);
    fprintf(GPLT, "%f %f\n%f %f\n\n", -xp, -yp, xp, -yp);
  }
  fprintf(GPLT, "e\n");
  while(fgets(buf, 80, stdin) != NULL) {
    if (strlen(buf) < 2 || buf[0] == '#') {
      fprintf(GPLT, "\n");
    } else {
      sscanf(buf, "%lf %lf", &x, &y);
      x *= M_PI/180;
      y *= M_PI/180;
      mollweide(&x, &y);
      fprintf(GPLT, "%f  %f\n", x, y);
    }
  }
  fprintf(GPLT, "e\n");
  fprintf(GPLT, "pause 30\n");
  fflush(GPLT);
  pclose(GPLT);

  return 0;
}
